package cn.iocoder.yudao.module.order.controller.admin.orderWarehouseIn;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseService;
import cn.iocoder.yudao.module.order.convert.orderWarehouseApproval.OrderWarehouseApprovalConvert;
import cn.iocoder.yudao.module.order.dal.dataobject.orderWarehouseApproval.OrderWarehouseApprovalDO;
import cn.iocoder.yudao.module.order.enums.OrderWarehouseApprovalTypeEnum;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import cn.iocoder.yudao.module.order.service.orderLabel.OrderLabelService;
import cn.iocoder.yudao.module.order.service.orderWarehouseAdjust.OrderWarehouseAdjustService;
import cn.iocoder.yudao.module.order.service.orderWarehouseApproval.OrderWarehouseApprovalService;
import cn.iocoder.yudao.module.order.service.orderWarehouseIn.OrderWarehouseInService;
import cn.iocoder.yudao.module.order.vo.order.OrderWarehouseInWithAttrVO;
import cn.iocoder.yudao.module.order.vo.order.WarehouseInInfoVO;
import cn.iocoder.yudao.module.order.vo.orderLabel.OrderLabelBackVO;
import cn.iocoder.yudao.module.order.vo.orderLabel.OrderLabelDto;
import cn.iocoder.yudao.module.order.vo.orderLabel.SaveOrUpdateOrderLabelDto;
import cn.iocoder.yudao.module.order.vo.orderWarehouseAdjust.*;
import cn.iocoder.yudao.module.order.vo.orderWarehouseApproval.OrderWarehouseApprovalBackVO;
import cn.iocoder.yudao.module.order.vo.orderWarehouseIn.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.text.MessageFormat;
import java.util.List;
import java.util.concurrent.TimeUnit;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.ORDER_FINISH_WAREHOUSE_IN_REPEAT_COMMIT;

@Validated
@RestController
@Api(tags = "管理后台 -  订单仓库相关接口")
@RequestMapping("/order/order-warehouse-in")
public class OrderWarehouseInController {

    @Resource
    private OrderWarehouseInService orderWarehouseInService;

    @Resource
    private OrderWarehouseApprovalService orderWarehouseApprovalService;

    @Resource
    private OrderWarehouseAdjustService warehouseAdjustService;

    @Resource
    private OrderService orderService;

    @Resource
    private WarehouseService warehouseService;

    @Resource
    private OrderLabelService orderLabelService;

    @Resource
    private RedisHelper redisHelper;

    /**
     * 订单完成入仓操作的放重复提交缓存key
     */
    private static final String ORDER_FINISH_WAREHOUSE_IN_KEY =
            "jiedao:order:finish:warehouse:in:{0}";


    /*
     创建入仓，保存入仓记录
     渠道，关联可出商品属性
     SELECT * from ecw_channel
     SELECT * from ecw_product_attr
     */
    @PostMapping("/create")
    @ApiOperation("入仓/追加")
    @Idempotent(timeout = 8)
    public CommonResult<Boolean> createOrderWarehouseIn(@Valid @RequestBody OrderWarehouseInCreateReqVO createReqVO) {
        return success(orderWarehouseInService.createOrderWarehouseIn(createReqVO));
    }


    @PostMapping("/delete")
    @ApiOperation("删除单次入仓纪录")
    public CommonResult<Boolean> delete(@RequestParam("orderWarehouseInId") Long orderWarehouseInId) {
        return success(orderWarehouseInService.deleteOrderWarehouseIn(orderWarehouseInId, false));
    }

    @PostMapping("/delete-when-packaging")
    @ApiOperation("删除单次入仓纪录-备货时")
    public CommonResult<Boolean> deleteWhenPacked(@RequestParam("orderWarehouseInId") Long orderWarehouseInId) {
        return success(orderWarehouseInService.deleteOrderWarehouseIn(orderWarehouseInId, true));
    }


    // @PutMapping("/update")
    // @ApiOperation("修改单次入仓纪录")
    // public CommonResult<Boolean> update(@Valid @RequestBody OrderWarehouseInUpdateReqVO updateReqVO) {
    //     // 校验存在
    //     orderWarehouseInService.updateOrderWarehouseIn(updateReqVO);
    //     return success(true);
    // }

    /*
     入仓记录修改
     */
    @PutMapping("/update-or-updateApply")
    @ApiOperation("入仓中直接入仓修改，完成入仓的发起修改请求")
    public CommonResult<Boolean> updateOrUpdateApply(@Valid @RequestBody OrderWarehouseInUpdateApplyReqVO updateReqVO) {
        // 校验存在
        String msg = orderWarehouseInService.updateOrUpdateApply(updateReqVO);
        CommonResult<Boolean> success = success(true);
        success.setMsg(msg);
        return success;
    }


    @GetMapping("/update-apply-callback")
    @ApiOperation("入仓修改-回调")
    public CommonResult<Boolean> updateApplyCallback(String formId, Integer result) {
        orderWarehouseInService.updateApplyCallback(formId, result);
        return success(true);
    }


    @PostMapping("/finish")
    @ApiOperation("入仓完成/转异")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> finishWarehouseIn(
            @Valid @RequestBody OrderWarehouseInFinishReqVO finishReqVO) {

        //完成入仓
        String redisKey = MessageFormat.format(ORDER_FINISH_WAREHOUSE_IN_KEY,
                finishReqVO.getOrderId().toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1) {
            return error(ORDER_FINISH_WAREHOUSE_IN_REPEAT_COMMIT);
        }
        boolean hasException = false;
        try {
            //完成入仓
            redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
            hasException = orderWarehouseInService.finishWarehouseIn(finishReqVO);

        } catch (Exception e) {
            throw e;
        } finally {
            redisHelper.delete(redisKey);
        }
        return success(hasException);
    }


    /**
     * 出货阶段 - 追加
     *
     * @param createReqVO
     * @return
     */
    @PostMapping("/append-apply-when-shipment")
    @ApiOperation("出货阶段 - 入仓/追加申请")
    @Idempotent(timeout = 8)
    public CommonResult<Boolean> appendApplyWhenShipment(@Valid @RequestBody OrderWarehouseInCreateReqVO createReqVO) {
        return success(orderWarehouseInService.appendApplyWhenShipment(createReqVO));
    }

    /*
     * 出货阶段-入仓修改申请
     */
    @PutMapping("/update-apply-when-shipment")
    @ApiOperation("出货阶段 - 入仓修改申请")
    public CommonResult<Boolean> updateApplyWhenShipment(@Valid @RequestBody OrderWarehouseInUpdateApplyReqVO updateReqVO) {
        // 校验存在
        String msg = orderWarehouseInService.updateApplyWhenShipment(updateReqVO);
        CommonResult<Boolean> success = success(true);
        success.setMsg(msg);
        return success;
    }


    @PostMapping("/save-or-update-order-location")
    @ApiOperation("新增/更新入仓储位")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> orderLocation(@Valid @RequestBody OrderWarehouseInOrderLocationReqVO orderLocationReqVO) {
        orderWarehouseInService.orderLocation(orderLocationReqVO);
        return success(true);
    }


    @GetMapping("/get-label-by-order")
    @ApiOperation("根据订单id获取已入箱的标签箱号列表")
    public CommonResult<List<OrderLabelBackVO>> getLabelByOrder(@RequestParam("orderId") Long orderId) {
        return success(orderLabelService.getLabelListByOrder(orderId));
    }


    @GetMapping("/get-label-wait-in-by-order")
    @ApiOperation("根据订单id获取待入仓标签箱号")
    public CommonResult<OrderLabelDto> getLabelWaitInByOrder(@RequestParam("orderId") Long orderId) {
        return success(orderLabelService.getLabelWaitInByOrder(orderId));
    }


    @PostMapping("/update-label")
    @ApiOperation("入仓修改箱号")
    public CommonResult<Boolean> updateLabel(@Valid @RequestBody SaveOrUpdateOrderLabelDto saveOrUpdateOrderLabelDto) {
        orderLabelService.saveOrUpdateLabel(saveOrUpdateOrderLabelDto);
        return success(true);
    }

    @GetMapping("/get-update-approval-info")
    @ApiOperation("获取入仓修改审批单详情-审批使用")
    @ApiImplicitParam(name = "formId", value = "流程id", required = true, example = "f622e0be-29ba-11ed-ba19-0242ac110002", dataTypeClass = String.class)
    public CommonResult<OrderWarehouseApprovalBackVO> getUpdateApprovalInfo(@RequestParam("formId") String formId) {
        // 校验存在
        return success(orderWarehouseApprovalService.getUpdateApprovalInfo(formId));
    }

    @GetMapping("/get-last-update-approval-info")
    @ApiOperation("获取最近的入仓修改审批单详情-修改页面使用")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId", value = "订单号", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "orderItemId", value = "订单项id", required = true, example = "1024", dataTypeClass = Long.class)
    })
    public CommonResult<OrderWarehouseApprovalBackVO> getLastUpdateApprovalInfo(@RequestParam("orderId") Long orderId, @RequestParam("orderItemId") Long orderItemId) {
        // 校验存在
        return success(orderWarehouseApprovalService.getLastUpdateApprovalInfo(orderId, orderItemId));
    }


    @GetMapping("/get-rollback-approval-info")
    @ApiOperation("退仓-审批单详情-后台使用")
    @ApiImplicitParam(name = "orderId", value = "订单id", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<OrderWarehouseApprovalBackVO> getRollbackApprovalInfo(@RequestParam("orderId") Long orderId) {
        // 校验存在
        OrderWarehouseApprovalDO orderWarehouseApprovalDO =
                orderWarehouseApprovalService.selectOne(OrderWarehouseApprovalTypeEnum.WAREHOUSE_IN_TO_RETURN.getValue(), orderId);

        return success(OrderWarehouseApprovalConvert.INSTANCE.convert(orderWarehouseApprovalDO));
    }

    @PutMapping("/rollback-order-item")
    @ApiOperation("退仓-订单项退仓（不审批）")
    public CommonResult<Boolean> deleteOrderWarehouseIn(@Valid @RequestBody OrderWarehouseInRollbackOrderItemReqVO orderItemReqVo) {
        orderWarehouseInService.deleteWarehouseInOrderItem(orderItemReqVo);
        return success(true);
    }

    @PutMapping("/rollback-apply")
    @ApiOperation("退仓-请求")
    public CommonResult<Boolean> orderWarehouseInRollbackApply(@Valid @RequestBody OrderWarehouseInRollbackApplyReqVO orderWarehouseInRollbackApplyReqVO) {
        orderWarehouseInService.rollbackApply(orderWarehouseInRollbackApplyReqVO, SecurityFrameworkUtils.getLoginUserId(), false);
        return success(true);
    }

    @PutMapping("/rollback-apply-callback")
    @ApiOperation("退仓-申请回调")
    public CommonResult<Boolean> orderWarehouseInRollbackApplyCallback(@RequestParam("approvalId") String approvalId, @RequestParam("result") Integer result) {
        orderWarehouseInService.rollbackApplyCallback(approvalId, result);
        return success(true);
    }

    @GetMapping("/get-order-warehouse-in")
    @ApiOperation("获得订单的入仓货物详情")
    @ApiImplicitParam(name = "orderId", value = "订单id", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<List<OrderWarehouseInBackVO>> getOrderWarehouseIn(@RequestParam("orderId") Long orderId) {
        List<OrderWarehouseInBackVO> orderWarehouseInBackVOList =
                orderWarehouseInService.getOrderWarehouseInList(orderId);
        return success(orderWarehouseInBackVOList);
    }

    @GetMapping("/get-order-warehouse-in-deleted")
    @ApiOperation("获得订单品名对应的退仓入仓货物详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId", value = "订单编号", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "orderItemId", value = "订单品名id", required = true, example = "1024", dataTypeClass = Long.class)
    })
    public CommonResult<OrderWarehouseInBackVO> getOrderWarehouseInDeleted(@RequestParam("orderId") Long orderId, @RequestParam("orderItemId") Long orderItemId) {
        OrderWarehouseInBackVO orderWarehouseInBackVODeleted =
                orderWarehouseInService.getOrderWarehouseInListDeleted(orderId, orderItemId);
        return success(orderWarehouseInBackVODeleted);
    }

    @PutMapping("/adjust-apply")
    @ApiOperation("调仓-发起请求")
    @Idempotent(timeout = 5)
    public CommonResult<String> adjustApply(@Valid @RequestBody OrderWarehouseAdjustCreateReqVO orderWarehouseAdjustCreateReqVO) {
        return success(warehouseAdjustService.adjustApply(orderWarehouseAdjustCreateReqVO));
    }

    @GetMapping("/adjust-apply-callback")
    @ApiOperation("调仓-回调")
    public CommonResult<Boolean> adjustApplyCallback(@RequestParam("bizId") String bizId, @RequestParam("result") Integer result) {
        warehouseAdjustService.adjustApplyCallback(bizId, result);
        return success(true);
    }

    @GetMapping("/get-adjust-apply-info")
    @ApiOperation("调仓-审批单详情-审批使用")
    @ApiImplicitParam(name = "formId", value = "审批表单formId", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<OrderWarehouseApprovalBackVO> getAdjustApplyInfo(@RequestParam("formId") Long formId) {
        return success(orderWarehouseApprovalService.getAdjustApprovalInfo(formId));
    }

    @GetMapping("/get-adjust-info")
    @ApiOperation("调仓-明细-后台使用")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId", value = "订单编号", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "lang", value = "语言 0中文 1英文", required = true, example = "0", dataTypeClass = Integer.class)
    })
    public CommonResult<List<OrderWarehouseAdjustBackVO>> getAdjustInfoByOrderId(@RequestParam("orderId") Long orderId, @RequestParam("lang") Integer lang) {
        return success(warehouseAdjustService.getWarehouseAdjustByOrderId(orderId, lang));
    }

    @PutMapping("/warehouse-adjust-send-out")
    @ApiOperation("调仓-调拨出仓")
    public CommonResult<Boolean> warehouseAdjustSendOut(@Valid @RequestBody OrderWarehouseAdjustSendOutVO sendOutVO) {
        warehouseAdjustService.updateWarehouseAdjustSendOut(sendOutVO);
        return success(true);
    }

    @PutMapping("/warehouse-adjust-send-out-batch")
    @ApiOperation("调仓-批量调拨出仓")
    public CommonResult<Boolean> warehouseAdjustSendOutBatch(@Valid @RequestBody OrderWarehouseAdjustSendOutBatchVO sendOutBatchVO) {
        warehouseAdjustService.updateWarehouseAdjustSendOutBatch(sendOutBatchVO);
        return success(true);
    }

    @PutMapping("/warehouse-adjust-arrived")
    @ApiOperation("调仓-调拨到仓")
    public CommonResult<Boolean> warehouseAdjustArrived(@Valid @RequestBody OrderWarehouseAdjustArrivedVO arrivedVO) {
        warehouseAdjustService.updateWarehouseAdjustArrived(arrivedVO);
        return success(true);
    }

    @GetMapping("/adjust-last-with-status")
    @ApiOperation("调仓-查看最近的调仓纪录和审核状态")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId", value = "订单编号", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "lang", value = "语言 0中文 1英文", required = true, example = "0", dataTypeClass = Integer.class)
    })
    public CommonResult<OrderWarehouseAdjustLastStatusBackVO> adjustLastWithStatus(@RequestParam("orderId") Long orderId, @RequestParam("lang") Integer lang) {
        return success(warehouseAdjustService.adjustLastWithStatus(orderId, lang));
    }

    @PostMapping("/cancel-adjust")
    @ApiOperation("调仓-取消调仓（出仓页面的取消调仓）")
    public CommonResult<Boolean> cancelAdjust(@Valid @RequestBody OrderWarehouseInAdjustCancelRequestVO cancelApplyReqVO) {
        //调仓-撤销
        return success(warehouseAdjustService.cancelAdjust(cancelApplyReqVO));
    }


    @PostMapping("/finish-packed")
    @ApiOperation("完成打包")
    @Idempotent(timeout = 2)
    public CommonResult<Boolean> finishPacked(@Valid @RequestBody OrderItemFinishPackedReqVO finishPackageReqVO) {
        orderWarehouseInService.finishPacked(finishPackageReqVO);
        return success(true);
    }


    @GetMapping("/no-need-to-pack")
    @ApiOperation("无需打包")
    @Idempotent(timeout = 2)
    public CommonResult<Boolean> noNeedToPack(@RequestParam("orderItemId") Long orderItemId) {
        orderWarehouseInService.noNeedToPack(orderItemId);
        return success(true);
    }

    @PostMapping("/no-need-to-packs")
    @ApiOperation("批量无需打包")
    @Idempotent(timeout = 2)
    public CommonResult<Boolean> noNeedToPacks(@RequestBody List<Long> orderItemIds) {
        orderWarehouseInService.noNeedToPacks(orderItemIds);
        return success(true);
    }

    @PutMapping("/finish-stocked")
    @ApiOperation("完成备货")
    @Idempotent(timeout = 2)
    public CommonResult<Boolean> finishStocked(@Valid @RequestBody OrderItemFinishStockedReqVO orderItemFinishStockedReqVO) {
        orderWarehouseInService.finishStocked(orderItemFinishStockedReqVO);
        return success(true);
    }


    @GetMapping("/statistics-first-finished-after-packed/{orderId}")
    @ApiOperation("获取打包后的订单首次完成入仓的统计数据")
    @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<WarehouseInInfoVO> getStatisticsFirstFinishedAfterPacked(@PathVariable("orderId") Long orderId) {
        return success(orderWarehouseInService.getStatisticsFirstFinishedAfterPacked(orderId));
    }


    @GetMapping("/get-order-warehouse-in-common-attr-list")
    @ApiOperation("获取订单入仓商品明细通用属性集合")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId", value = "订单号", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "excludeOrderWarehouseInId", value = "需要排除的入仓id", example = "1024", dataTypeClass = Long.class)
    })
    public CommonResult<List<OrderWarehouseInWithAttrVO>> getOrderWarehouseInCommonAttrList(@RequestParam("orderId") Long orderId,
                                                                                            @RequestParam("excludeOrderWarehouseInId") Long excludeOrderWarehouseInId) {
        return success(orderWarehouseInService.getOrderWarehouseInCommonAttrList(orderId, excludeOrderWarehouseInId));
    }

    /**
     * 有订单，但客户类型不正确。故这里通过客户订单来修改客户状态
     *
     * @return
     */
    @GetMapping("/change-customer-status-by-order")
    public CommonResult<Boolean> test() {

        orderWarehouseInService.changeCustomerStatusByOrder();

        return success(true);
    }

}
