package cn.iocoder.yudao.module.order.controller.app.order;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.file.FileNameUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.util.spring.enums.UserTypeEnum;
import cn.iocoder.yudao.framework.common.exception.ErrorCode;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.code.CodeUtils;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.object.JavaBeanUtils;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.limiter.dynamic.DynamicRateLimiter;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.framework.velocity.VelocityHelper;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import cn.iocoder.yudao.module.order.convert.order.OrderConvert;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.dto.OrderBackInfoDto;
import cn.iocoder.yudao.module.order.dto.PackingListDto;
import cn.iocoder.yudao.module.order.enums.OrderTempLateEnum;
import cn.iocoder.yudao.module.order.service.order.OrderItemService;
import cn.iocoder.yudao.module.order.service.order.OrderQueryService;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import cn.iocoder.yudao.module.order.vo.order.*;
import cn.iocoder.yudao.module.order.vo.orderItem.ItemPackingListVO;
import cn.iocoder.yudao.module.order.vo.orderTime.OrderTimeBackVO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductDO;
import cn.iocoder.yudao.module.product.service.product.ProductService;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.pdf.PdfWriter;
import com.itextpdf.tool.xml.XMLWorkerHelper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.*;
import java.math.BigDecimal;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.text.MessageFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.enums.GlobalErrorCodeConstants.UNAUTHORIZED;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.*;

@Validated
@RestController
@Api(tags = "web端 - app - 我的订单")
@RequestMapping("/my/order")
@Slf4j
public class MyOrderController {

    @Autowired
    private OrderService orderService;
    @Autowired
    private OrderQueryService orderQueryService;
    @Autowired
    private OrderItemService orderItemService;
    @Autowired
    private CustomerContactsService customerContactsService;

    @Autowired
    private MemberUserService memberUserService;


    @Autowired
    private ProductService productService;
    @Autowired
    private DictDataApi dictDataApi;
    @Autowired
    private FileService fileService;

    @Value("${ue.templates-url}")
    private String path;

    @Value("${ue.temp-dir}")
    private String tempDir;

    @Resource
    private RedisHelper redisHelper;


    /**
     * 订单修改操作的防重复提交缓存key
     */
    private static final String ORDER_UPDATE_KEY = "jiedao:order:update:{0}";

    @PostMapping("/create")
    @DynamicRateLimiter(base = "#Headers['Authorization']", permits = 1)
    @ApiOperation(value = "创建订单", notes = "发货人只取当前用户信息，收货人取当前用户的客户信息")
    @Idempotent(timeout = 15)
    @PreAuthenticated
    public CommonResult<Long> createOrder(@Valid @RequestBody OrderCreateReqVO createReqVO) {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        Integer userType = loginUser.getUserType();
        if (userType != 1){
            return error(UNAUTHORIZED);
        }
        Long userId = loginUser.getId();
        createReqVO.setUserId(userId);
        MemberUserDO memberUserDO = memberUserService.info(userId);
        if (Objects.isNull(memberUserDO) ||
                memberUserDO.getStatus() == 1 ||
                memberUserDO.getDeleted()) {
            return error(MEMBER_NOT_EXISTS);
        }
        if (null == createReqVO.getStatus()) {
            createReqVO.setStatus(2);
        }
        if (StringUtils.isNotBlank(createReqVO.getType()) &&
                createReqVO.getType().contains("4")) {
            // TODO 客户端下单处理未知的类型
            String type = Arrays.stream(createReqVO.getType().split(StrUtil.COMMA)).filter(s -> !StringUtils.equals("4", s)).filter(StringUtils::isNotBlank).collect(Collectors.joining(StrUtil.COMMA));
            createReqVO.setType(type);
        }
        OrderKycReqVO kycReqVO = new OrderKycReqVO();
        kycReqVO.setControlCargoStatus(createReqVO.getIsCargoControl());
        kycReqVO.setLineId(createReqVO.getLineId());
        orderQueryService.checkKycStatus(kycReqVO, userId);
        return success(orderService.createOrder(createReqVO, memberUserDO).getOrderId());
    }

    @PostMapping("/copy/{copyOrderId}")
    @DynamicRateLimiter(base = "#Headers['Authorization']", permits = 1)
    @ApiOperation(value = "复制订单", notes = "发货人只取当前用户信息，收货人取当前用户的客户信息")
    @Idempotent(timeout = 15)
    @PreAuthenticated
    public CommonResult<Long> copyOrder(@PathVariable Long copyOrderId) {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        Integer userType = loginUser.getUserType();
        if (userType != 1){
            return error(UNAUTHORIZED);
        }
        Long userId = loginUser.getId();
        MemberUserDO memberUserDO = memberUserService.info(userId);
        if (Objects.isNull(memberUserDO) ||
                memberUserDO.getStatus() == 1 ||
                memberUserDO.getDeleted()) {
            return error(MEMBER_NOT_EXISTS);
        }
        Long orderId = orderService.copyOrder(copyOrderId, memberUserDO);
        return success(orderId);
    }

    @PostMapping("/checkKycStatus")
    @ApiOperation("检查kyc状态")
    public CommonResult<Boolean> checkKycStatus(OrderKycReqVO kycReqVO) {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        Integer userType = loginUser.getUserType();
        if (userType != 1){
            return error(UNAUTHORIZED);
        }
        Long userId = loginUser.getId();
        return success(orderQueryService.checkKycStatus(kycReqVO,userId));
    }


    @GetMapping("/getKycStatus")
    @ApiOperation("获取kyc状态")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "lineId", value = "线路ID", required = true, example = "3", dataType = "Long"),
            @ApiImplicitParam(name = "controlCargoStatus", value = "是否控货", required = true, example = "false", dataType = "Boolean")
    })
    public CommonResult<OrderKycRespVo> getKycStatus(@RequestParam(value = "lineId") Long lineId, @RequestParam(value = "controlCargoStatus") Boolean controlCargoStatus) {
        OrderKycReqVO kycReqVO = new OrderKycReqVO();
        kycReqVO.setLineId(lineId);
        kycReqVO.setControlCargoStatus(controlCargoStatus);
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        Integer userType = loginUser.getUserType();
        if (userType != 1){
            return error(UNAUTHORIZED);
        }
        Long userId = loginUser.getId();
        return success(orderQueryService.getKycStatus(kycReqVO,userId));
    }

    @PutMapping("/update")
    @ApiOperation(value = "更新订单", notes = "发货人只取当前用户信息，收货人取当前用户的客户信息")
    @PreAuthenticated
    public CommonResult<Boolean> updateOrder(@Valid @RequestBody OrderUpdateReqVO updateReqVO) {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        Integer userType = loginUser.getUserType();
        if (userType != 1){
            return error(UNAUTHORIZED);
        }
        Long userId = loginUser.getId();
        updateReqVO.setUserId(userId);
        MemberUserDO memberUserDO = memberUserService.info(userId);
        if (StringUtils.isNotBlank(updateReqVO.getType()) && updateReqVO.getType().contains("4")) {
            // TODO 客户端下单处理未知的类型
            String type = Arrays.stream(updateReqVO.getType().split(StrUtil.COMMA)).filter(s -> !StringUtils.equals("4", s)).filter(StringUtils::isNotBlank).collect(Collectors.joining(StrUtil.COMMA));
            updateReqVO.setType(type);
        }
//        if (Objects.isNull(memberUserDO) || memberUserDO.getStatus() == 1 || memberUserDO.getDeleted()) {
//            return error(MEMBER_NOT_EXISTS);
//        }
        String redisKey = MessageFormat.format(ORDER_UPDATE_KEY, updateReqVO.getOrderId().toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1){
            return error(ORDER_UPDATE_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
        orderService.updateOrder(updateReqVO, memberUserDO);
        redisHelper.delete(redisKey);
        return success(true);
    }

    @PutMapping("/update/apply")
    @ApiOperation("(新修改订单)订单修改审批申请")
    @PreAuthenticated
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> specialApplyOrder(@Valid @RequestBody OrderUpdateReqVO updateReqVO) {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        Integer userType = loginUser.getUserType();
        if (userType != 1){
            return error(UNAUTHORIZED);
        }
        Long userId = loginUser.getId();
        updateReqVO.setUserId(userId);
        MemberUserDO memberUserDO = memberUserService.info(userId);
        if (Objects.isNull(memberUserDO) || memberUserDO.getStatus() == 1 || memberUserDO.getDeleted()) {
            return error(MEMBER_NOT_EXISTS);
        }
        String redisKey = MessageFormat.format(ORDER_UPDATE_KEY, updateReqVO.getOrderId().toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1){
            return error(ORDER_UPDATE_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
        orderService.updateOrderApply(updateReqVO, memberUserDO);
        redisHelper.delete(redisKey);
        return success(true);
    }


    @GetMapping("update/info/{orderId}")
    @ApiOperation("(新获取订单编辑详情)根据订单ID获得订单修改申请详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId", value = "该值为orderId", required = true, example = "1024", dataType = "Long"),
            @ApiImplicitParam(name = "isResubmit", value = "是否重新提交获取 true/false", required = false, example = "false", dataType = "Boolean")
    })
//    @PreAuthorize("@ss.hasPermission('ecw:order-approval:query')")
    public CommonResult<OrderBackVO> getUpdateOrderApplyInfoByOrderId(@PathVariable(value = "orderId") Long orderId,
                                                                      @RequestParam(value = "isResubmit", defaultValue = "false") boolean isResubmit) {
        OrderBackVO orderBackVO = orderQueryService.getUpdateOrderApplyInfo(orderId, isResubmit);
        return success(orderBackVO);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除订单")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<Boolean> deleteOrder(@RequestParam("orderId") Long orderId) {
        String redisKey = MessageFormat.format(ORDER_UPDATE_KEY, orderId.toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1){
            return error(ORDER_UPDATE_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
        orderService.deleteOrder(orderId);
        redisHelper.delete(redisKey);
        return success(true);
    }

    @PutMapping("/recovery/{orderId}")
    @ApiOperation("恢复订单")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<Boolean> recoveryOrder(@PathVariable("orderId") Long orderId) {
        String redisKey = MessageFormat.format(ORDER_UPDATE_KEY, orderId.toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1){
            return error(ORDER_UPDATE_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
        orderService.recoveryOrder(orderId);
        redisHelper.delete(redisKey);
        return success(true);
    }

    @PutMapping("/cancel/{orderId}")
    @ApiOperation("取消订单")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<Boolean> cancelOrder(@PathVariable("orderId") Long orderId) {
        String redisKey = MessageFormat.format(ORDER_UPDATE_KEY, orderId.toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1){
            return error(ORDER_UPDATE_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
        orderService.cancelOrder(orderId);
        redisHelper.delete(redisKey);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得编辑详情订单")
    @PreAuthenticated
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<OrderBackVO> getOrder(@RequestParam("orderId") Long orderId) {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        Integer userType = loginUser.getUserType();
        if (userType != 1){
            return error(UNAUTHORIZED);
        }
        Long userId = loginUser.getId();
        OrderBackVO orderBackVO = orderQueryService.getOrder(orderId);
        orderBackVO.setUserType(UserTypeEnum.MEMBER.getValue());
//        if (orderBackVO.getUserId() == null && !Objects.equals(orderBackVO.getUserId(), userId)) {
//            return error(THIS_ORDER_IS_NOT_YOUR);
//        }
        return success(orderBackVO);
    }

    @GetMapping("/info/{orderId}")
    @ApiOperation("订单详情")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<OrderBackInfoDto> info(@PathVariable("orderId") Long orderId) {
        OrderBackInfoDto dto = orderQueryService.info(orderId);
        dto.setUserType(UserTypeEnum.MEMBER.getValue());
        if (CollectionUtil.isNotEmpty(dto.getOrderTimeVOList())) {
            List<OrderTimeBackVO> orderTimeVOList = dto.getOrderTimeVOList().stream()
                    .collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(OrderTimeBackVO::getStatusType))), ArrayList::new));
            dto.setOrderTimeVOList(orderTimeVOList.stream().filter(t -> t.getStatus() != 15 && t.getStatus() != 150330).sorted(Comparator.comparing(OrderTimeBackVO::getId).reversed()).collect(Collectors.toList()));
        }
        return success(dto);
    }


    @GetMapping("/dynamic/page")
    @ApiOperation("获得我的订单动态分页")
    @PreAuthenticated
    public CommonResult<PageResult<OrderBackPageVO>> dynamicOrderPage(OrderQueryVO query, PageVO page) {
        query.setUserType(UserTypeEnum.MEMBER.getValue());
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        Integer userType = loginUser.getUserType();
        if (!Objects.equals(userType, UserTypeEnum.MEMBER.getValue())){
            return error(UNAUTHORIZED);
        }
        Long userId = loginUser.getId();
        query.setUserId(userId);
        // 根据会员ID获取客户联系人ID与客户ID
        CustomerContactsDO customerContacts = customerContactsService.getOne(new LambdaQueryWrapper<CustomerContactsDO>()
                .eq(CustomerContactsDO::getUserid, userId)
                .eq(CustomerContactsDO::getDeleted, 0)
                .last("limit 1"));
        // 根据会员ID获取客户手机号
        MemberUserDO memberUserDO = memberUserService.info(userId);
        if (Objects.nonNull(customerContacts)) {
            query.setConsignorContactsId(customerContacts.getId());
        }
        if (Objects.nonNull(memberUserDO)) {
            query.setConsignorPhone(memberUserDO.getMobile());
        }
        // TODO 后台以会员关联的联系人下单订单数据是否返回
        PageResult<OrderBackPageVO> pageResult = orderQueryService.dynamicOrderPage(query, page);
        if (CollectionUtil.isNotEmpty(pageResult.getList())) {
            for (OrderBackPageVO orderBackPageVO : pageResult.getList()) {
                if ((Objects.nonNull(orderBackPageVO.getUserId()) && Objects.equals(userId, orderBackPageVO.getUserId()))) {
                    orderBackPageVO.setDeliverGoods(Boolean.TRUE);
                }
                if ((StringUtils.isNotBlank(orderBackPageVO.getConsignorPhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getConsignorPhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getConsignorCountryCode()))) {
                    orderBackPageVO.setDeliverGoods(Boolean.TRUE);
                }
                if (StringUtils.isNotBlank(orderBackPageVO.getConsigneePhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getConsigneePhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getConsigneeCountryCode())) {
                    orderBackPageVO.setReceiving(Boolean.TRUE);
                }
                if (StringUtils.isNotBlank(orderBackPageVO.getCargoControlPhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getCargoControlPhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getCargoControlCountryCode())) {
                    orderBackPageVO.setCargoController(Boolean.TRUE);
                }
            }
        }
        return success(pageResult);
    }


    @GetMapping("/issued/page")
    @ApiOperation("获得我发出的订单分页")
    @PreAuthenticated
    public CommonResult<PageResult<OrderBackPageVO>> issuedOrderPage(OrderQueryVO query, PageVO page) {
        query.setUserType(UserTypeEnum.MEMBER.getValue());
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        Integer userType = loginUser.getUserType();
        if (!Objects.equals(userType, UserTypeEnum.MEMBER.getValue())){
            return error(UNAUTHORIZED);
        }
        Long userId = loginUser.getId();
        query.setUserId(userId);
        // 根据会员ID获取客户联系人ID与客户ID
        List<CustomerContactsDO> customerContactsList = customerContactsService.list(new LambdaQueryWrapper<CustomerContactsDO>()
                .eq(CustomerContactsDO::getUserid, userId)
                .eq(CustomerContactsDO::getDeleted, 0));
        // 根据会员ID获取客户手机号
        MemberUserDO memberUserDO = memberUserService.info(userId);
        if (CollectionUtil.isNotEmpty(customerContactsList)) {
            Set<Long> contactsIds = customerContactsList.stream().map(CustomerContactsDO::getId).collect(Collectors.toSet());
            query.setConsignorContactsIds(contactsIds);
        }
        if (Objects.nonNull(memberUserDO)) {
            query.setConsignorPhone(memberUserDO.getMobile());
        }
        // TODO 后台以会员关联的联系人下单订单数据是否返回
        PageResult<OrderBackPageVO> pageResult = orderQueryService.myIssuedOrderPage(query, page);
        if (CollectionUtil.isNotEmpty(pageResult.getList())) {
            for (OrderBackPageVO orderBackPageVO : pageResult.getList()) {
                if ((Objects.nonNull(orderBackPageVO.getUserId()) && Objects.equals(userId, orderBackPageVO.getUserId()))) {
                    orderBackPageVO.setDeliverGoods(Boolean.TRUE);
                }
                if ((StringUtils.isNotBlank(orderBackPageVO.getConsignorPhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getConsignorPhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getConsignorCountryCode()))) {
                    orderBackPageVO.setDeliverGoods(Boolean.TRUE);
                }
                if (StringUtils.isNotBlank(orderBackPageVO.getConsigneePhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getConsigneePhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getConsigneeCountryCode())) {
                    orderBackPageVO.setReceiving(Boolean.TRUE);
                }
                if (StringUtils.isNotBlank(orderBackPageVO.getCargoControlPhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getCargoControlPhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getCargoControlCountryCode())) {
                    orderBackPageVO.setCargoController(Boolean.TRUE);
                }
            }
        }
        return success(pageResult);
    }


    @GetMapping("/received/page")
    @ApiOperation("获得我收到的订单分页")
    @PreAuthenticated
    public CommonResult<PageResult<OrderBackPageVO>> receivedOrderPage(OrderQueryVO query, PageVO page) {
        query.setUserType(UserTypeEnum.MEMBER.getValue());
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        Integer userType = loginUser.getUserType();
        if (!Objects.equals(userType, UserTypeEnum.MEMBER.getValue())){
            return error(UNAUTHORIZED);
        }
        Long userId = loginUser.getId();
        // 根据会员ID获取客户联系人ID与客户ID
        List<CustomerContactsDO> customerContactsList = customerContactsService.list(new LambdaQueryWrapper<CustomerContactsDO>()
                .eq(CustomerContactsDO::getUserid, userId)
                .eq(CustomerContactsDO::getDeleted, 0));
        // 根据会员ID获取客户手机号
        MemberUserDO memberUserDO = memberUserService.info(userId);
        if (CollectionUtil.isNotEmpty(customerContactsList)) {
            Set<Long> contactsIds = customerContactsList.stream().map(CustomerContactsDO::getId).collect(Collectors.toSet());
            query.setConsigneeContactsIds(contactsIds);
        }
        if (Objects.nonNull(memberUserDO)) {
            query.setConsigneePhone(memberUserDO.getMobile());
        }
        if (null == query.getConsigneeContactsId() && StringUtils.isBlank(query.getConsigneePhone())) {
            return success(new PageResult<>());
        }
        // TODO 暂时用当前用户关联的客户联系人ID 和 当前用户手机号去匹配订单收货人的客户联系人ID与手机号
        PageResult<OrderBackPageVO> pageResult = orderQueryService.receivedOrderPage(query, page);
        if (CollectionUtil.isNotEmpty(pageResult.getList())) {
            for (OrderBackPageVO orderBackPageVO : pageResult.getList()) {
                if ((Objects.nonNull(orderBackPageVO.getUserId()) && Objects.equals(userId, orderBackPageVO.getUserId()))) {
                    orderBackPageVO.setDeliverGoods(Boolean.TRUE);
                }
                if ((StringUtils.isNotBlank(orderBackPageVO.getConsignorPhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getConsignorPhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getConsignorCountryCode()))) {
                    orderBackPageVO.setDeliverGoods(Boolean.TRUE);
                }
                if (StringUtils.isNotBlank(orderBackPageVO.getConsigneePhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getConsigneePhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getConsigneeCountryCode())) {
                    orderBackPageVO.setReceiving(Boolean.TRUE);
                }
                if (StringUtils.isNotBlank(orderBackPageVO.getCargoControlPhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getCargoControlPhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getCargoControlCountryCode())) {
                    orderBackPageVO.setCargoController(Boolean.TRUE);
                }
            }
        }
        return success(pageResult);
    }

    @GetMapping("/cargo/control/page")
    @ApiOperation("获得我控货的订单分页")
    @PreAuthenticated
    public CommonResult<PageResult<OrderBackPageVO>> cargoControlOrderPage(OrderQueryVO query, PageVO page) {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        Integer userType = loginUser.getUserType();
        if (!Objects.equals(userType, UserTypeEnum.MEMBER.getValue())){
            return error(UNAUTHORIZED);
        }
        Long userId = loginUser.getId();

        // 根据会员ID获取客户联系人ID与客户ID
//        CustomerContactsDO customerContacts = customerContactsService.getOne(new LambdaQueryWrapper<CustomerContactsDO>()
//                .eq(CustomerContactsDO::getUserid, userId)
//                .eq(CustomerContactsDO::getDeleted, 0)
//                .last("limit 1"));
        // 根据会员ID获取客户手机号
        MemberUserDO memberUserDO = memberUserService.info(userId);
//        if (Objects.nonNull(customerContacts)) {
//            query.setConsignorContactsId(customerContacts.getId());
//        }
        if (Objects.nonNull(memberUserDO)) {
//            query.setConsignorPhone(memberUserDO.getMobile());
            query.setCargoControlPhone(memberUserDO.getMobile());
        }
        if (Objects.isNull(query) || (null == query.getConsigneeContactsId() && StringUtils.isBlank(query.getCargoControlPhone()))) {
            return success(new PageResult<>());
        }
        query.setUserType(UserTypeEnum.MEMBER.getValue());
        // TODO 暂时用当前用户关联的客户联系人ID 和 当前用户手机号去匹配订单发货人的客户联系人ID与手机号
        // TODO 控货人信息待管理业务晚上后加入搜索条件
        PageResult<OrderBackPageVO> pageResult = orderQueryService.appCargoControlOrderPage(query, page);
        if (CollectionUtil.isNotEmpty(pageResult.getList())) {
            for (OrderBackPageVO orderBackPageVO : pageResult.getList()) {
                if ((Objects.nonNull(orderBackPageVO.getUserId()) && Objects.equals(userId, orderBackPageVO.getUserId()))) {
                    orderBackPageVO.setDeliverGoods(Boolean.TRUE);
                }
                if ((StringUtils.isNotBlank(orderBackPageVO.getConsignorPhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getConsignorPhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getConsignorCountryCode()))) {
                    orderBackPageVO.setDeliverGoods(Boolean.TRUE);
                }
                if (StringUtils.isNotBlank(orderBackPageVO.getConsigneePhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getConsigneePhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getConsigneeCountryCode())) {
                    orderBackPageVO.setReceiving(Boolean.TRUE);
                }
                if (StringUtils.isNotBlank(orderBackPageVO.getCargoControlPhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getCargoControlPhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getCargoControlCountryCode())) {
                    orderBackPageVO.setCargoController(Boolean.TRUE);
                }
            }
        }
        return success(pageResult);
    }


    @GetMapping("/all/order")
    @ApiOperation("app-全部的订单分页")
    @PreAuthenticated
    public CommonResult<PageResult<OrderBackPageVO>> myAllOrderPage(OrderQueryVO query, PageVO page) {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        Integer userType = loginUser.getUserType();
        if (!Objects.equals(userType, UserTypeEnum.MEMBER.getValue())){
            return error(UNAUTHORIZED);
        }
        Long userId = loginUser.getId();
        if (Objects.isNull(query)) {
            query = new OrderQueryVO();
        }
        query.setUserId(userId);
        query.setUserType(UserTypeEnum.MEMBER.getValue());
        // 根据会员ID获取客户联系人ID与客户ID
        List<CustomerContactsDO> customerContactsList = customerContactsService.list(new LambdaQueryWrapper<CustomerContactsDO>()
                .eq(CustomerContactsDO::getUserid, userId)
                .eq(CustomerContactsDO::getDeleted, 0));
        // 根据会员ID获取客户手机号
        MemberUserDO memberUserDO = memberUserService.info(userId);
        if (CollectionUtil.isNotEmpty(customerContactsList)) {
            Set<Long> contactsIds = customerContactsList.stream().map(CustomerContactsDO::getId).collect(Collectors.toSet());
            query.setConsigneeContactsIds(contactsIds);
            query.setConsignorContactsIds(contactsIds);
        }
        if (Objects.nonNull(memberUserDO)) {
            query.setConsigneePhone(memberUserDO.getMobile());
            query.setConsignorPhone(memberUserDO.getMobile());
            query.setCargoControlPhone(memberUserDO.getMobile());
        }
        if (null == query.getConsigneeContactsId() && StringUtils.isBlank(query.getConsigneePhone())) {
            return success(new PageResult<>());
        }
        // TODO 暂时用当前用户关联的客户联系人ID 和 当前用户手机号去匹配订单发货人的客户联系人ID与手机号
        // TODO 控货人信息待管理业务晚上后加入搜索条件
        PageResult<OrderBackPageVO> pageResult = orderQueryService.myAllOrderPage(query, page);
        if (CollectionUtil.isNotEmpty(pageResult.getList())) {
            for (OrderBackPageVO orderBackPageVO : pageResult.getList()) {
                if ((Objects.nonNull(orderBackPageVO.getUserId()) && Objects.equals(userId, orderBackPageVO.getUserId()))) {
                    orderBackPageVO.setDeliverGoods(Boolean.TRUE);
                }
                if ((StringUtils.isNotBlank(orderBackPageVO.getConsignorPhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getConsignorPhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getConsignorCountryCode()))) {
                    orderBackPageVO.setDeliverGoods(Boolean.TRUE);
                }
                if (StringUtils.isNotBlank(orderBackPageVO.getConsigneePhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getConsigneePhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getConsigneeCountryCode())) {
                    orderBackPageVO.setReceiving(Boolean.TRUE);
                }
                if (StringUtils.isNotBlank(orderBackPageVO.getCargoControlPhone()) && StringUtils.equals(memberUserDO.getMobile(), orderBackPageVO.getCargoControlPhone()) && StringUtils.equals(memberUserDO.getAreaCode(), orderBackPageVO.getCargoControlCountryCode())) {
                    orderBackPageVO.setCargoController(Boolean.TRUE);
                }
            }
        }
        return success(pageResult);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出订单 Excel")
    @OperateLog(type = EXPORT)
    @PreAuthenticated
    public void exportOrderExcel(@Valid OrderQueryVO query,
                                 HttpServletResponse response) throws IOException {
        List<OrderDO> list = orderQueryService.getOrderList(query);
        // 导出 Excel
        List<OrderBackVO> datas = OrderConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "订单.xls", "数据", OrderBackVO.class, datas);
    }


    @GetMapping("/billLading/template")
    @ApiOperation("获取提单pdf")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "billLadingNo", value = "提单编号", required = true, example = "TD20220717", dataTypeClass = String.class),
            @ApiImplicitParam(name = "type", value = "类型:1 海运 2 海空联运 3 空运", required = true, example = "1", dataTypeClass = Integer.class)
    })
    @PreAuthenticated
    @OperateLog(type = EXPORT)
    @Deprecated
    public CommonResult<String> getBillLadingTemplatePDF(String billLadingNo, Integer type, HttpServletResponse response) throws IOException, DocumentException {
        // 根据类型不同取不同的模板
        OrderTempLateEnum anEnum = OrderTempLateEnum.getEnumByCode(type);
        if (Objects.isNull(anEnum)) {
            return error(new ErrorCode(100000, "类型不存在"));
        }
        String templateName = I18nMessage.getLang() == 0 ? anEnum.getNameZh() : anEnum.getNameEn();
        //设置打印后pdf的格式
        Document document = new Document(PageSize.A4, 60.5f, 60.5f, 33f, 33f);

        PdfWriter pdfWriter = PdfWriter.getInstance(document, response.getOutputStream());// outputStream可以自己指定位置或者是HttpServletResponse得到的输出流
        pdfWriter.setTagged();
        document.open();
        //取得velocity的上下文
        //往vm中写入信息
        //写入自己要插入的信息，
        // TODO 根据提单查询 订单详情
        Map<String, Object> param = new HashMap<>();
        param.put("billLadingNo", "TD20220719E");
        param.put("orderNo", "CNG20220701N");
        param.put("consigneeName", "张三");
        param.put("plNo", "CNG20220701N-01");
        param.put("consigneePhone", "+13289227288");
        param.put("consignorPhone", "+8615677282285");
        param.put("consignorName", "李四");
        param.put("deliveryDate", "2022-07-01");

        // TODO 测试使用指定的测试数据
        List<OrderItemDO> orderItemDOList = orderItemService.list(new LambdaQueryWrapper<OrderItemDO>()
                .eq(OrderItemDO::getOrderNo, "NSA2200001L"));
        if (CollectionUtil.isEmpty(orderItemDOList)) {
            return error(1000000, "该提单没有商品信息");
        }
        List<Map<String, Object>> mapList = JavaBeanUtils.objectsToMaps(orderItemDOList);

        param.put("totalAmount", "66668.99");
        //TODO 读取源文件,未避免服务器上文件路径读取问题,这里直接获取流文件,放入新创建的临时文件中进行处理
        InputStream inputStream = getClass().getClassLoader().getResourceAsStream(path + "/" + templateName + ".html");
        if (inputStream == null) {
            return error(1000000, "未找到模板文件");
        }
        File file = new File(tempDir + "/" + "copy_" + templateName + ".html");
        FileUtils.copyToFile(inputStream, file);
        Writer writer = VelocityHelper.getWriterByTemplate(file, null, param, mapList);
        //输出转输入流
        InputStream in = new ByteArrayInputStream(writer.toString().getBytes(Charset.defaultCharset()));
        //去找velocity并交给itext生成pdf
        XMLWorkerHelper.getInstance().parseXHtml(pdfWriter, document, in, StandardCharsets.UTF_8);
        FileUtils.delete(file);// TODO 删除临时文件
        inputStream.close(); // 关闭文件流
        in.close();
        document.close();

        return success("");
    }

    @PostMapping("/import")
    @ApiOperation("导入装箱单")
    @ApiImplicitParam(name = "file", value = "Excel 文件", required = true, dataTypeClass = MultipartFile.class)
    @PreAuthenticated
    public CommonResult<PackingListDto> importExcel(@RequestParam("file") MultipartFile file) throws Exception {
        List<ItemPackingListVO> list = ExcelUtils.read(file, ItemPackingListVO.class);
        if (CollectionUtil.isNotEmpty(list)) {
            for (ItemPackingListVO itemPackingListVO : list) {
                itemPackingListVO.setBrandType(Objects.equals("y", itemPackingListVO.getBrandTitle()) ? 1 : 0);
                ProductDO productDO = productService.findProdByTitle(itemPackingListVO.getProdTitleZh(), itemPackingListVO.getProdTitleEn());
                if (Objects.nonNull(productDO)) {
                    itemPackingListVO.setProdId(productDO.getId());
                }
            }
            String fix = FileNameUtil.getSuffix(file.getOriginalFilename());
            String path = DateUtils.getDir() + IdUtil.objectId() + (StrUtil.isBlank(fix) ? StrUtil.EMPTY : StrUtil.DOT + fix);
            String url = fileService.createFile(path, file.getBytes());
            PackingListDto dto = new PackingListDto();
            dto.setPackingListUrl(url);
            dto.setList(list.stream().filter(vo -> Objects.nonNull(vo.getProdId())).collect(Collectors.toList()));
            return success(dto);
        }
        return error(PACKING_LIST_NOT_READ_DATA);
    }

    @GetMapping("/export-packing-list")
    @ApiOperation("获取装箱单模板")
    public void exportExcel(HttpServletResponse response) throws IOException {
        // 随机获取数据库10条可用数据
        List<ProductDO> productDOList = productService.randomTemplateProdData(10);
        List<DictDataRespDTO> dictDataRespDTOList = dictDataApi.getDictDatas("product_material");
        Map<String, DictDataRespDTO> dictDataRespDTOMap = dictDataRespDTOList.stream().collect(Collectors.toMap(DictDataRespDTO::getValue, Function.identity(), (key1, key2) -> key2));
        List<ItemPackingListVO> list = new ArrayList<>();
        for (int i = 0; i < productDOList.size(); i++) {
            ProductDO productDO = productDOList.get(i);
            list.add(ItemPackingListVO.builder()
                    .id(i + 1).num(1)
                    .brandTitle(i % 2 == 0 ? "n" : "y").prodTitleZh(productDO.getTitleZh()).prodTitleEn(productDO.getTitleEn())
                    .lessNum(1).quantity(1).volume(BigDecimal.ONE).weight(BigDecimal.ONE)
                    .material(I18nMessage.getLang() == 0 ? dictDataRespDTOMap.get(productDO.getMaterialType()).getLabel() : dictDataRespDTOMap.get(productDO.getMaterialType()).getLabelEn())
                    .build());
        }
        ExcelUtils.write(response, I18nMessage.getLang() == 0 ? "装箱单模板.xls" : "packing-list-template.xls", "数据", ItemPackingListVO.class, list);
    }


    @GetMapping("/courier/number")
    @ApiOperation("生成快递单号")
    public CommonResult<String> getCourierNumber() {
        // TODO 业务待完善
        return success(CodeUtils.getCourierNumber());
    }


//    @GetMapping("/refresh-customer-performance")
//    @ApiOperation(value = "更新订单发货人与收货人信息，且更新业绩归属", notes = "示例：NG2300208L,NG2300212L,NG2300209L多个订单编号逗号连接")
//    public void updateOrderCustomer(@RequestParam(value = "orderNumbers")String orderNumbers)  {
//        List<String> list = new ArrayList<>();
//        if (StringUtils.isNotBlank(orderNumbers)){
//            list.addAll(Arrays.asList(orderNumbers.split(StrUtil.COMMA)));
//        }
//        orderService.updateOrderCustomer(list);
//    }
}
